Certificate Request:
    Data:
        Version: 0 (0x0)
        Subject: C=AU, O=Dodgy Brothers, CN=Dodgy CA
        Subject Public Key Info:
            Public Key Algorithm: rsaEncryption
            Public Key: (512 bit)
                Modulus:
                    00:c8:dd:f8:ef:ac:5c:65:ab:a9:c8:2f:65:90:7f:
                    6e:03:c7:95:2d:ce:8c:bb:b1:68:6f:4c:dd:1d:82:
                    fe:66:a0:77:60:70:75:c5:78:3d:a6:5c:e5:8c:38:
                    88:35:f2:e4:ab:4c:ed:d1:15:a8:21:b7:ce:b7:8c:
                    82:72:c5:d4:ff
                Exponent: 65537 (0x10001)
        Attributes:
            a0:00
    Signature Algorithm: md5withRSAEncryption
        26:a6:d7:f5:06:e3:e0:d9:94:e9:5d:b9:99:08:51:93:7f:57:
        40:15:f4:48:67:db:39:d9:36:e4:47:f9:d8:27:d1:eb:c0:76:
        6c:04:ee:01:4d:34:dc:3e:8b:8f:80:87:07:84:31:3d:70:d8:
        cc:d1:9d:ca:b4:a1:37:36:01:49
-----BEGIN CERTIFICATE REQUEST-----
MIHxMIGcAgEAMDkxCzAJBgNVBAYTAkFVMRcwFQYDVQQKEw5Eb2RneSBCcm90aGVy
czERMA8GA1UEAxMIRG9kZ3kgQ0EwWjALBgkqhkiG9w0BAQEDSwAwSAJBAMjd+O+s
XGWrqcgvZZB/bgPHlS3OjLuxaG9M3R2C/magd2BwdcV4PaZc5Yw4iDXy5KtM7dEV
qCG3zreMgnLF1P8CAwEAAaAAMA0GCSqGSIb3DQEBBAUAA0EAJqbX9Qbj4NmU6V25
mQhRk39XQBX0SGfbOdk25Ef52CfR68B2bATuAU003D6Lj4CHB4QxPXDYzNGdyrSh
NzYBSQ==
-----END CERTIFICATE REQUEST-----
