;
; Definition file for the DDL version of the CRYPT16 library from SSLeay
;

LIBRARY         CRYPT16

DESCRIPTION     'SSLeay CRYPT16 - eay@mincom.oz.au'

CODE            PRELOAD MOVEABLE
DATA            PRELOAD MOVEABLE SINGLE

EXETYPE		WINDOWS

HEAPSIZE	4096
STACKSIZE	8192

EXPORTS
    _SSLeay                             @1
    _SSLeay_version                     @2
    _ASN1_BIT_STRING_asn1_meth          @600
    _ASN1_HEADER_free                   @6
    _ASN1_HEADER_new                    @7
    _ASN1_IA5STRING_asn1_meth           @601
    _ASN1_INTEGER_get                   @8
    _ASN1_INTEGER_set                   @9
    _ASN1_INTEGER_to_BN                 @10
    _ASN1_OBJECT_free                   @11
    _ASN1_OBJECT_new                    @12
    _ASN1_PRINTABLE_type                @13
    _ASN1_STRING_cmp                    @602
    _ASN1_STRING_dup                    @603
    _ASN1_STRING_free                   @604
    _ASN1_STRING_new                    @605
    _ASN1_STRING_print                  @606
    _ASN1_STRING_set                    @607
    _ASN1_STRING_type_new               @608
    _ASN1_TYPE_free                     @15
    _ASN1_TYPE_new                      @16
    _ASN1_UTCTIME_check                 @17
    _ASN1_UTCTIME_print                 @21
    _ASN1_UTCTIME_set                   @22
    _ASN1_check_infinite_end            @24
    _ASN1_d2i_bio                       @25
    _ASN1_digest                        @638
    _ASN1_dup                           @27
    _ASN1_get_object                    @28
    _ASN1_i2d_bio                       @29
    _ASN1_object_size                   @31
    _ASN1_parse                         @32
    _ASN1_put_object                    @33
    _ASN1_sign                          @34
    _ASN1_verify                        @35
    _BF_cbc_encrypt                     @651
    _BF_cfb64_encrypt                   @652
    _BF_ecb_encrypt                     @653
    _BF_encrypt                         @654
    _BF_ofb64_encrypt                   @655
    _BF_options                         @657
    _BF_set_key                         @656
    _BIO_ctrl                           @36
    _BIO_ctrl_int                       @598
    _BIO_debug_callback                 @37
    _BIO_dump                           @640
    _BIO_f_base64                       @615
    _BIO_f_buffer                       @38
    _BIO_f_cipher                       @616
    _BIO_f_md                           @39
    _BIO_fd_should_retry                @637
    _BIO_find_type                      @650
    _BIO_free                           @40
    _BIO_free_all                       @649
    _BIO_gets                           @41
    _BIO_new                            @42
    _BIO_pop                            @648
    _BIO_printf                         @639
    _BIO_push                           @647
    _BIO_puts                           @43
    _BIO_read                           @44
    _BIO_s_fd                           @45
    _BIO_s_file_internal_w16            @573
    _BIO_s_mem                          @47
    _BIO_s_null                         @48
    _BIO_s_socket                       @49
    _BIO_set                            @588
    _BIO_set_cipher                     @614
    _BIO_sock_should_retry              @636
    _BIO_write                          @50
    _BN_CTX_free                        @51
    _BN_CTX_new                         @52
    _BN_MONT_CTX_free                   @53
    _BN_MONT_CTX_new                    @54
    _BN_MONT_CTX_set                    @55
    _BN_add                             @56
    _BN_add_word                        @57
    _BN_bin2bn                          @58
    _BN_bn2ascii                        @59
    _BN_bn2bin                          @60
    _BN_clear                           @61
    _BN_clear_bit                       @62
    _BN_clear_free                      @63
    _BN_cmp                             @64
    _BN_copy                            @65
    _BN_div                             @66
    _BN_dup                             @67
    _BN_free                            @68
    _BN_from_montgomery                 @69
    _BN_gcd                             @70
    _BN_generate_prime                  @71
    _BN_is_bit_set                      @72
    _BN_is_prime                        @73
    _BN_lshift                          @74
    _BN_lshift1                         @75
    _BN_mask_bits                       @76
    _BN_mod                             @77
    _BN_mod_exp                         @78
    _BN_mod_exp_mont                    @79
    _BN_mod_exp_recp                    @80
    _BN_mod_exp_simple                  @81
    _BN_mod_inverse                     @82
    _BN_mod_mul                         @83
    _BN_mod_mul_montgomery              @84
    _BN_mod_mul_reciprocal              @85
    _BN_mod_word                        @86
    _BN_mul                             @87
    _BN_new                             @88
    _BN_num_bits                        @89
    _BN_num_bits_word                   @90
    _BN_options                         @91
    _BN_print                           @92
    _BN_rand                            @94
    _BN_reciprocal                      @95
    _BN_rshift                          @96
    _BN_rshift1                         @97
    _BN_set_bit                         @98
    _BN_set_word                        @99
    _BN_sqr                             @100
    _BN_sub                             @101
    _BN_to_ASN1_INTEGER                 @102
    _BN_ucmp                            @103
    _BN_value_one                       @104
    _BUF_MEM_free                       @105
    _BUF_MEM_grow                       @106
    _BUF_MEM_new                        @107
    _CERTIFICATE_CTX_free               @108
    _CERTIFICATE_CTX_new                @109
    _CONF_free                          @110
    _CONF_get_number                    @111
    _CONF_get_section                   @112
    _CONF_get_string                    @113
    _CONF_load                          @114
    _CRYPTO_add_lock                    @582
    _CRYPTO_dbg_free                    @645
    _CRYPTO_dbg_malloc                  @643
    _CRYPTO_dbg_realloc                 @644
    _CRYPTO_dbg_remalloc                @596
    _CRYPTO_free                        @591
    _CRYPTO_get_add_lock_callback       @580
    _CRYPTO_get_id_callback             @115
    _CRYPTO_get_lock_name               @583
    _CRYPTO_get_locking_callback        @116
    _CRYPTO_get_mem_functions           @642
    _CRYPTO_lock                        @117
    _CRYPTO_malloc                      @589
    _CRYPTO_mem_ctrl                    @592
    _CRYPTO_mem_leaks                   @593
    _CRYPTO_mem_leaks_cb                @646
    _CRYPTO_realloc                     @590
    _CRYPTO_remalloc                    @595
    _CRYPTO_set_add_lock_callback       @581
    _CRYPTO_set_id_callback             @118
    _CRYPTO_set_locking_callback        @119
    _CRYPTO_set_mem_functions           @641
    _CRYPTO_thread_id                   @120
    _DH_check                           @121
    _DH_compute_key                     @122
    _DH_free                            @123
    _DH_generate_key                    @124
    _DH_generate_parameters             @125
    _DH_new                             @126
    _DH_size                            @127
    _DHparams_print                     @128
    _DSA_free                           @130
    _DSA_new                            @131
    _DSA_print                          @132
    _DSA_sign                           @133
    _DSA_size                           @134
    _DSA_verify                         @135
    _ERR_clear_error                    @136
    _ERR_error_string                   @137
    _ERR_free_strings                   @138
    _ERR_func_error_string              @139
    _ERR_get_err_state_table            @140
    _ERR_get_error                      @141
    _ERR_get_error_line                 @142
    _ERR_get_state                      @143
    _ERR_get_string_table               @144
    _ERR_lib_error_string               @145
    _ERR_load_ASN1_strings              @146
    _ERR_load_BN_strings                @147
    _ERR_load_BUF_strings               @148
    _ERR_load_CONF_strings              @149
    _ERR_load_DH_strings                @150
    _ERR_load_DSA_strings               @151
    _ERR_load_ERR_strings               @152
    _ERR_load_EVP_strings               @153
    _ERR_load_OBJ_strings               @155
    _ERR_load_PEM_strings               @156
    _ERR_load_RSA_strings               @157
    _ERR_load_X509_strings              @158
    _ERR_load_crypto_strings            @159
    _ERR_load_strings                   @160
    _ERR_peek_error                     @161
    _ERR_peek_error_line                @162
    _ERR_print_errors                   @163
    _ERR_put_error                      @165
    _ERR_reason_error_string            @166
    _ERR_remove_state                   @167
    _EVP_BytesToKey                     @168
    _EVP_CipherFinal                    @169
    _EVP_CipherInit                     @170
    _EVP_CipherUpdate                   @171
    _EVP_DecodeBlock                    @172
    _EVP_DecodeFinal                    @173
    _EVP_DecodeInit                     @174
    _EVP_DecodeUpdate                   @175
    _EVP_DecryptFinal                   @176
    _EVP_DecryptInit                    @177
    _EVP_DecryptUpdate                  @178
    _EVP_DigestFinal                    @179
    _EVP_DigestInit                     @180
    _EVP_DigestUpdate                   @181
    _EVP_EncodeBlock                    @182
    _EVP_EncodeFinal                    @183
    _EVP_EncodeInit                     @184
    _EVP_EncodeUpdate                   @185
    _EVP_EncryptFinal                   @186
    _EVP_EncryptInit                    @187
    _EVP_EncryptUpdate                  @188
    _EVP_OpenFinal                      @189
    _EVP_OpenInit                       @190
    _EVP_PKEY_free                      @191
    _EVP_PKEY_new                       @192
    _EVP_PKEY_size                      @193
    _EVP_SealFinal                      @194
    _EVP_SealInit                       @195
    _EVP_SignFinal                      @196
    _EVP_VerifyFinal                    @197
    _EVP_bf_cbc                         @658
    _EVP_bf_cfb                         @659
    _EVP_bf_ecb                         @660
    _EVP_bf_ofb                         @661
    _EVP_des_cbc                        @198
    _EVP_des_cfb                        @199
    _EVP_des_ecb                        @200
    _EVP_des_ede                        @201
    _EVP_des_ede3                       @202
    _EVP_des_ede3_cbc                   @203
    _EVP_des_ede3_cfb                   @204
    _EVP_des_ede3_ofb                   @205
    _EVP_des_ede_cbc                    @206
    _EVP_des_ede_cfb                    @207
    _EVP_des_ede_ofb                    @208
    _EVP_des_ofb                        @209
    _EVP_desx_cbc                       @586
    _EVP_dss                            @584
    _EVP_dss1                           @585
    _EVP_get_MDbyname                   @210
    _EVP_get_cipherbyname               @211
    _EVP_get_pw_prompt                  @212
    _EVP_idea_cbc                       @213
    _EVP_idea_cfb                       @214
    _EVP_idea_ecb                       @215
    _EVP_idea_ofb                       @216
    _EVP_md2                            @217
    _EVP_md5                            @218
    _EVP_rc2_cbc                        @219
    _EVP_rc2_cfb                        @220
    _EVP_rc2_ecb                        @221
    _EVP_rc2_ofb                        @222
    _EVP_rc4                            @223
    _EVP_read_pw_string                 @224
    _EVP_set_pw_prompt                  @225
    _EVP_sha                            @226
    _EVP_sha1                           @227
    _MD2                                @228
    _MD2_Final                          @229
    _MD2_Init                           @230
    _MD2_Update                         @231
    _MD2_options                        @232
    _MD5                                @233
    _MD5_Final                          @234
    _MD5_Init                           @235
    _MD5_Update                         @236
    _NETSCAPE_SPKAC_free                @237
    _NETSCAPE_SPKAC_new                 @238
    _NETSCAPE_SPKI_free                 @239
    _NETSCAPE_SPKI_new                  @240
    _OBJ_bsearch                        @241
    _OBJ_cmp                            @242
    _OBJ_dup                            @243
    _OBJ_ln2nid                         @244
    _OBJ_nid2ln                         @245
    _OBJ_nid2obj                        @246
    _OBJ_nid2sn                         @247
    _OBJ_obj2nid                        @248
    _OBJ_sn2nid                         @249
    _OBJ_txt2nid                        @250
    _PEM_ASN1_read_bio                  @252
    _PEM_ASN1_write_bio                 @254
    _PEM_SealFinal                      @255
    _PEM_SealInit                       @256
    _PEM_SealUpdate                     @257
    _PEM_SignFinal                      @258
    _PEM_X509_INFO_read_bio             @260
    _PEM_do_header                      @261
    _PEM_get_EVP_CIPHER_INFO            @262
    _PEM_read_bio                       @264
    _PEM_write_bio                      @266
    _PKCS7_DIGEST_free                  @267
    _PKCS7_DIGEST_new                   @268
    _PKCS7_ENCRYPT_free                 @269
    _PKCS7_ENCRYPT_new                  @270
    _PKCS7_ENC_CONTENT_free             @271
    _PKCS7_ENC_CONTENT_new              @272
    _PKCS7_ENVELOPE_free                @273
    _PKCS7_ENVELOPE_new                 @274
    _PKCS7_ISSUER_AND_SERIAL_free       @275
    _PKCS7_ISSUER_AND_SERIAL_new        @276
    _PKCS7_RECIP_INFO_free              @277
    _PKCS7_RECIP_INFO_new               @278
    _PKCS7_SIGNED_free                  @279
    _PKCS7_SIGNED_new                   @280
    _PKCS7_SIGNER_INFO_free             @281
    _PKCS7_SIGNER_INFO_new              @282
    _PKCS7_SIGN_ENVELOPE_free           @283
    _PKCS7_SIGN_ENVELOPE_new            @284
    _PKCS7_free                         @285
    _PKCS7_new                          @286
    _RAND_bytes                         @287
    _RAND_cleanup                       @288
    _RAND_file_name                     @289
    _RAND_load_file                     @290
    _RAND_screen                        @613
    _RAND_seed                          @291
    _RAND_write_file                    @292
    _RC2_cbc_encrypt                    @293
    _RC2_cfb64_encrypt                  @294
    _RC2_ecb_encrypt                    @295
    _RC2_encrypt                        @296
    _RC2_ofb64_encrypt                  @297
    _RC2_set_key                        @298
    _RC4                                @299
    _RC4_options                        @300
    _RC4_set_key                        @301
    _RSAPrivateKey_asn1_meth            @609
    _RSA_free                           @302
    _RSA_generate_key                   @303
    _RSA_mod_exp                        @304
    _RSA_new                            @305
    _RSA_print                          @306
    _RSA_private_decrypt                @308
    _RSA_private_encrypt                @309
    _RSA_public_decrypt                 @310
    _RSA_public_encrypt                 @311
    _RSA_sign                           @312
    _RSA_size                           @313
    _RSA_verify                         @314
    _SHA                                @315
    _SHA1                               @316
    _SHA1_Final                         @317
    _SHA1_Init                          @318
    _SHA1_Update                        @319
    _SHA_Final                          @320
    _SHA_Init                           @321
    _SHA_Update                         @322
    _TXT_DB_create_index                @323
    _TXT_DB_free                        @324
    _TXT_DB_get_by_index                @325
    _TXT_DB_insert                      @326
    _TXT_DB_read                        @327
    _TXT_DB_write                       @328
    _X509_ALGOR_free                    @329
    _X509_ALGOR_new                     @330
    _X509_ATTRIBUTE_free                @331
    _X509_ATTRIBUTE_new                 @332
    _X509_CINF_free                     @333
    _X509_CINF_new                      @334
    _X509_CRL_INFO_free                 @335
    _X509_CRL_INFO_new                  @336
    _X509_CRL_free                      @337
    _X509_CRL_new                       @338
    _X509_EXTENSION_free                @577
    _X509_EXTENSION_new                 @576
    _X509_INFO_free                     @339
    _X509_INFO_new                      @340
    _X509_NAME_ENTRY_free               @341
    _X509_NAME_ENTRY_new                @342
    _X509_NAME_free                     @343
    _X509_NAME_new                      @344
    _X509_NAME_oneline                  @345
    _X509_NAME_print                    @346
    _X509_PKEY_free                     @347
    _X509_PKEY_new                      @348
    _X509_PUBKEY_free                   @349
    _X509_PUBKEY_new                    @350
    _X509_REQ_INFO_free                 @351
    _X509_REQ_INFO_new                  @352
    _X509_REQ_free                      @354
    _X509_REQ_get_pubkey                @622
    _X509_REQ_new                       @355
    _X509_REQ_print                     @356
    _X509_REQ_set_pubkey                @621
    _X509_REQ_set_subject_name          @618
    _X509_REQ_set_version               @619
    _X509_REQ_to_X509                   @358
    _X509_REVOKED_free                  @359
    _X509_REVOKED_new                   @360
    _X509_SIG_free                      @361
    _X509_SIG_new                       @362
    _X509_VAL_free                      @363
    _X509_VAL_new                       @364
    _X509_add_cert                      @365
    _X509_add_cert_dir                  @366
    _X509_add_cert_file                 @367
    _X509_asn1_meth                     @610
    _X509_cert_verify                   @368
    _X509_cert_verify_error_string      @369
    _X509_check_private_key             @599
    _X509_cmp_current_time              @370
    _X509_free                          @372
    _X509_get_cert                      @373
    _X509_get_default_cert_area         @374
    _X509_get_default_cert_dir          @375
    _X509_get_default_cert_dir_env      @376
    _X509_get_default_cert_file         @377
    _X509_get_default_cert_file_env     @378
    _X509_get_default_private_dir       @379
    _X509_get_issuer_name               @630
    _X509_get_pubkey                    @632
    _X509_get_subject_name              @628
    _X509_gmtime_adj                    @382
    _X509_issuer_and_serial_cmp         @383
    _X509_issuer_and_serial_hash        @384
    _X509_issuer_name_cmp               @385
    _X509_issuer_name_hash              @386
    _X509_load_verify_locations         @387
    _X509_name_cmp                      @388
    _X509_name_hash                     @389
    _X509_new                           @390
    _X509_print                         @391
    _X509_set_default_verify_paths      @393
    _X509_set_issuer_name               @629
    _X509_set_notAfter                  @634
    _X509_set_notBefore                 @635
    _X509_set_pubkey                    @631
    _X509_set_serialNumber              @623
    _X509_set_subject_name              @627
    _X509_set_version                   @625
    _X509_subject_name_cmp              @394
    _X509_subject_name_hash             @395
    _X509_to_X509_REQ                   @396
    _a2i_ASN1_INTEGER                   @397
    _a2i_ASN1_STRING                    @611
    _asn1_Finish                        @398
    _asn1_GetSequence                   @399
    _bn_div64                           @400
    _bn_expand2                         @401
    _bn_mul_add_word                    @402
    _bn_mul_word                        @403
    _bn_qadd                            @597
    _bn_qsub                            @404
    _bn_sqr_words                       @405
    _crypt                              @406
    _d2i_ASN1_BIT_STRING                @407
    _d2i_ASN1_BOOLEAN                   @574
    _d2i_ASN1_HEADER                    @408
    _d2i_ASN1_IA5STRING                 @409
    _d2i_ASN1_INTEGER                   @410
    _d2i_ASN1_OBJECT                    @411
    _d2i_ASN1_OCTET_STRING              @412
    _d2i_ASN1_PRINTABLE                 @413
    _d2i_ASN1_PRINTABLESTRING           @414
    _d2i_ASN1_SET                       @415
    _d2i_ASN1_T61STRING                 @416
    _d2i_ASN1_TYPE                      @417
    _d2i_ASN1_UTCTIME                   @418
    _d2i_ASN1_bytes                     @419
    _d2i_DHparams                       @420
    _d2i_DSAPrivateKey                  @421
    _d2i_DSAPublicKey                   @422
    _d2i_NETSCAPE_SPKAC                 @423
    _d2i_NETSCAPE_SPKI                  @424
    _d2i_Netscape_RSA                   @425
    _d2i_PKCS7                          @426
    _d2i_PKCS7_DIGEST                   @427
    _d2i_PKCS7_ENCRYPT                  @428
    _d2i_PKCS7_ENC_CONTENT              @429
    _d2i_PKCS7_ENVELOPE                 @430
    _d2i_PKCS7_ISSUER_AND_SERIAL        @431
    _d2i_PKCS7_RECIP_INFO               @432
    _d2i_PKCS7_SIGNED                   @433
    _d2i_PKCS7_SIGNER_INFO              @434
    _d2i_PKCS7_SIGN_ENVELOPE            @435
    _d2i_PrivateKey                     @436
    _d2i_PublicKey                      @437
    _d2i_RSAPrivateKey                  @438
    _d2i_RSAPublicKey                   @439
    _d2i_X509                           @440
    _d2i_X509_ALGOR                     @441
    _d2i_X509_ATTRIBUTE                 @442
    _d2i_X509_CINF                      @443
    _d2i_X509_CRL                       @444
    _d2i_X509_CRL_INFO                  @445
    _d2i_X509_EXTENSION                 @579
    _d2i_X509_NAME                      @446
    _d2i_X509_NAME_ENTRY                @447
    _d2i_X509_PKEY                      @448
    _d2i_X509_PUBKEY                    @449
    _d2i_X509_REQ                       @450
    _d2i_X509_REQ_INFO                  @451
    _d2i_X509_REVOKED                   @452
    _d2i_X509_SIG                       @453
    _d2i_X509_VAL                       @454
    _d2i_asn1_print_type                @455
    _des_3cbc_encrypt                   @456
    _des_cbc_cksum                      @457
    _des_cbc_encrypt                    @458
    _des_cblock_print_file              @459
    _des_cfb64_encrypt                  @460
    _des_cfb_encrypt                    @461
    _des_decrypt3                       @633
    _des_ecb3_encrypt                   @462
    _des_ecb_encrypt                    @463
    _des_ede3_cbc_encrypt               @464
    _des_ede3_cfb64_encrypt             @465
    _des_ede3_ofb64_encrypt             @466
    _des_enc_read                       @467
    _des_enc_write                      @468
    _des_encrypt                        @469
    _des_encrypt2                       @470
    _des_encrypt3                       @617
    _des_fcrypt                         @612
    _des_is_weak_key                    @471
    _des_key_sched                      @472
    _des_ncbc_encrypt                   @473
    _des_ofb64_encrypt                  @474
    _des_ofb_encrypt                    @475
    _des_options                        @476
    _des_pcbc_encrypt                   @477
    _des_quad_cksum                     @478
    _des_random_key                     @479
    _des_random_seed                    @480
    _des_read_2passwords                @481
    _des_read_password                  @482
    _des_read_pw_string                 @483
    _des_set_key                        @484
    _des_set_odd_parity                 @485
    _des_string_to_2keys                @486
    _des_string_to_key                  @487
    _des_xcbc_encrypt                   @587
    _i2a_ASN1_INTEGER                   @488
    _i2a_ASN1_OBJECT                    @489
    _i2d_ASN1_BIT_STRING                @490
    _i2d_ASN1_BOOLEAN                   @575
    _i2d_ASN1_HEADER                    @491
    _i2d_ASN1_IA5STRING                 @492
    _i2d_ASN1_INTEGER                   @493
    _i2d_ASN1_OBJECT                    @494
    _i2d_ASN1_OCTET_STRING              @495
    _i2d_ASN1_PRINTABLE                 @496
    _i2d_ASN1_SET                       @497
    _i2d_ASN1_TYPE                      @498
    _i2d_ASN1_UTCTIME                   @499
    _i2d_ASN1_bytes                     @500
    _i2d_DHparams                       @501
    _i2d_NETSCAPE_SPKAC                 @502
    _i2d_NETSCAPE_SPKI                  @503
    _i2d_Netscape_RSA                   @504
    _i2d_PKCS7                          @505
    _i2d_PKCS7_DIGEST                   @506
    _i2d_PKCS7_ENCRYPT                  @507
    _i2d_PKCS7_ENC_CONTENT              @508
    _i2d_PKCS7_ENVELOPE                 @509
    _i2d_PKCS7_ISSUER_AND_SERIAL        @510
    _i2d_PKCS7_RECIP_INFO               @511
    _i2d_PKCS7_SIGNED                   @512
    _i2d_PKCS7_SIGNER_INFO              @513
    _i2d_PKCS7_SIGN_ENVELOPE            @514
    _i2d_PrivateKey                     @515
    _i2d_PublicKey                      @516
    _i2d_RSAPrivateKey                  @517
    _i2d_RSAPublicKey                   @518
    _i2d_X509                           @519
    _i2d_X509_ALGOR                     @520
    _i2d_X509_ATTRIBUTE                 @521
    _i2d_X509_CINF                      @522
    _i2d_X509_CRL                       @523
    _i2d_X509_CRL_INFO                  @524
    _i2d_X509_EXTENSION                 @578
    _i2d_X509_NAME                      @525
    _i2d_X509_NAME_ENTRY                @526
    _i2d_X509_PKEY                      @527
    _i2d_X509_PUBKEY                    @528
    _i2d_X509_REQ                       @529
    _i2d_X509_REQ_INFO                  @530
    _i2d_X509_REVOKED                   @531
    _i2d_X509_SIG                       @532
    _i2d_X509_VAL                       @533
    _idea_cbc_encrypt                   @534
    _idea_cfb64_encrypt                 @535
    _idea_ecb_encrypt                   @536
    _idea_encrypt                       @537
    _idea_ofb64_encrypt                 @538
    _idea_options                       @539
    _idea_set_decrypt_key               @540
    _idea_set_encrypt_key               @541
    _lh_delete                          @542
    _lh_doall                           @543
    _lh_doall_arg                       @544
    _lh_free                            @545
    _lh_insert                          @546
    _lh_new                             @547
    _lh_node_stats_bio                  @549
    _lh_node_usage_stats_bio            @551
    _lh_retrieve                        @552
    _lh_stats_bio                       @554
    _lh_strhash                         @555
    _sk_delete                          @559
    _sk_delete_ptr                      @560
    _sk_find                            @561
    _sk_free                            @562
    _sk_insert                          @563
    _sk_new                             @564
    _sk_pop                             @565
    _sk_pop_free                        @566
    _sk_push                            @567
    _sk_shift                           @568
    _sk_unshift                         @569

