;
; Definition file for the DDL version of the SSL32 library from SSLeay
;

LIBRARY         SSL32

DESCRIPTION     'SSLeay SSL32 - eay@mincom.oz.au'

EXPORTS
    BIO_f_ssl                          @1
    ERR_load_SSL_strings               @2
    SSL2_accept                        @14
    SSL2_connect                       @17
    SSL2_peek                          @76
    SSL2_read                          @35
    SSL2_write                         @58
    SSL_CTX_check_private_key          @74
    SSL_CTX_free                       @3
    SSL_CTX_new                        @4
    SSL_CTX_set_cert_verify_cb         @5
    SSL_CTX_set_cipher_list            @6
    SSL_CTX_set_default_passwd_cb      @72
    SSL_CTX_set_verify                 @7
    SSL_CTX_use_PrivateKey             @61
    SSL_CTX_use_PrivateKey_ASN1        @62
    SSL_CTX_use_PrivateKey_file        @63
    SSL_CTX_use_RSAPrivateKey          @64
    SSL_CTX_use_RSAPrivateKey_ASN1     @65
    SSL_CTX_use_RSAPrivateKey_file     @66
    SSL_CTX_use_certificate            @67
    SSL_CTX_use_certificate_ASN1       @68
    SSL_CTX_use_certificate_file       @69
    SSL_ERR                            @8
    SSL_LOG                            @9
    SSL_SESSION_free                   @10
    SSL_SESSION_new                    @11
    SSL_SESSION_print                  @12
    SSL_SESSION_print_fp               @13
    SSL_add_session                    @15
    SSL_check_private_key              @75
    SSL_clear                          @16
    SSL_copy_session_id                @18
    SSL_debug                          @19
    SSL_flush_sessions                 @20
    SSL_free                           @21
    SSL_get_cipher                     @22
    SSL_get_cipher_bits                @73
    SSL_get_cipher_list                @23
    SSL_get_fd                         @24
    SSL_get_peer_certificate           @25
    SSL_get_rbio                       @26
    SSL_get_read_ahead                 @27
    SSL_get_shared_ciphers             @28
    SSL_get_time                       @29
    SSL_get_timeout                    @30
    SSL_get_wbio                       @31
    SSL_load_error_strings             @32
    SSL_new                            @33
    SSL_pending                        @34
    SSL_remove_session                 @36
    SSL_rstate_string                  @37
    SSL_rstate_string_long             @38
    SSL_set_bio                        @39
    SSL_set_cipher_list                @40
    SSL_set_fd                         @41
    SSL_set_read_ahead                 @42
    SSL_set_rfd                        @70
    SSL_set_session                    @43
    SSL_set_time                       @44
    SSL_set_timeout                    @45
    SSL_set_verify                     @46
    SSL_set_wfd                        @71
    SSL_state_string                   @47
    SSL_state_string_long              @48
    SSL_use_PrivateKey                 @49
    SSL_use_PrivateKey_ASN1            @50
    SSL_use_PrivateKey_file            @51
    SSL_use_RSAPrivateKey              @52
    SSL_use_RSAPrivateKey_ASN1         @53
    SSL_use_RSAPrivateKey_file         @54
    SSL_use_certificate                @55
    SSL_use_certificate_ASN1           @56
    SSL_use_certificate_file           @57
    d2i_SSL_SESSION                    @59
    i2d_SSL_SESSION                    @60

