-----BEGIN CERTIFICATE-----
MIIBXTCCAQcCAQYwDQYJKoZIhvcNAQEEBQAwOTELMAkGA1UEBhMCQVUxFzAVBgNV
BAoTDkRvZGd5IEJyb3RoZXJzMREwDwYDVQQDEwhEb2RneSBDQTAeFw05NzAxMTMx
OTI0MTlaFw05NzAyMTIxOTI0MTlaMDoxCzAJBgNVBAYTAkFVMRcwFQYDVQQKEw5E
b2RneSBCcm90aGVyczESMBAGA1UEAxMJQSBCcm90aGVyMFwwDQYJKoZIhvcNAQEB
BQADSwAwSAJBAPaBQYFz0isIIkhQPxzo6aB1i35VwtpuzKSMb60lx3FsSvo3dGUD
T3LlYrrT9Pki+ILirfcI64WB66OYBfi8CoUCAwEAATANBgkqhkiG9w0BAQQFAANB
AFbI+u4/ASVElQjOF7AePDuIDdL0LJKIs8an1QvvjEqdopa9s51ogjiyuu+UfvZN
wMq5+YvOW/sWMaL1ZvYGKX8=
-----END CERTIFICATE-----
