FLAGS="-DTERMIO -O3 -DB_ENDIAN -fomit-frame-pointer -mv8 -Wall -Iinclude"
SHFLAGS="-DPIC -fpic"

gcc -shared -Icrypto $SHFLAGS $FLAGS -o libcrypto.so crypto/crypto.c -lnsl -lsocket
gcc -shared -Issl $SHFLAGS $FLAGS -o libssl.so ssl/ssl.c -L. -lcrypto
