;
; Definition file for the DDL version of the SSL16 library from SSLeay
;

LIBRARY         SSL16

DESCRIPTION     'SSLeay SSL16 - eay@mincom.oz.au'

CODE            PRELOAD MOVEABLE
DATA            PRELOAD MOVEABLE SINGLE

EXETYPE		WINDOWS

HEAPSIZE	4096
STACKSIZE	8192

EXPORTS
    _BIO_f_ssl                          @1
    _ERR_load_SSL_strings               @2
    _SSL2_accept                        @14
    _SSL2_connect                       @17
    _SSL2_peek                          @76
    _SSL2_read                          @35
    _SSL2_write                         @58
    _SSL_CTX_check_private_key          @74
    _SSL_CTX_free                       @3
    _SSL_CTX_new                        @4
    _SSL_CTX_set_cert_verify_cb         @5
    _SSL_CTX_set_cipher_list            @6
    _SSL_CTX_set_default_passwd_cb      @72
    _SSL_CTX_set_verify                 @7
    _SSL_CTX_use_PrivateKey             @61
    _SSL_CTX_use_PrivateKey_ASN1        @62
    _SSL_CTX_use_PrivateKey_file        @63
    _SSL_CTX_use_RSAPrivateKey          @64
    _SSL_CTX_use_RSAPrivateKey_ASN1     @65
    _SSL_CTX_use_RSAPrivateKey_file     @66
    _SSL_CTX_use_certificate            @67
    _SSL_CTX_use_certificate_ASN1       @68
    _SSL_CTX_use_certificate_file       @69
    _SSL_ERR                            @8
    _SSL_LOG                            @9
    _SSL_SESSION_free                   @10
    _SSL_SESSION_new                    @11
    _SSL_SESSION_print                  @12
    _SSL_add_session                    @15
    _SSL_check_private_key              @75
    _SSL_clear                          @16
    _SSL_copy_session_id                @18
    _SSL_debug                          @19
    _SSL_flush_sessions                 @20
    _SSL_free                           @21
    _SSL_get_cipher                     @22
    _SSL_get_cipher_bits                @73
    _SSL_get_cipher_list                @23
    _SSL_get_fd                         @24
    _SSL_get_peer_certificate           @25
    _SSL_get_rbio                       @26
    _SSL_get_read_ahead                 @27
    _SSL_get_shared_ciphers             @28
    _SSL_get_time                       @29
    _SSL_get_timeout                    @30
    _SSL_get_wbio                       @31
    _SSL_load_error_strings             @32
    _SSL_new                            @33
    _SSL_pending                        @34
    _SSL_remove_session                 @36
    _SSL_rstate_string                  @37
    _SSL_rstate_string_long             @38
    _SSL_set_bio                        @39
    _SSL_set_cipher_list                @40
    _SSL_set_fd                         @41
    _SSL_set_read_ahead                 @42
    _SSL_set_rfd                        @70
    _SSL_set_session                    @43
    _SSL_set_time                       @44
    _SSL_set_timeout                    @45
    _SSL_set_verify                     @46
    _SSL_set_wfd                        @71
    _SSL_state_string                   @47
    _SSL_state_string_long              @48
    _SSL_use_PrivateKey                 @49
    _SSL_use_PrivateKey_ASN1            @50
    _SSL_use_PrivateKey_file            @51
    _SSL_use_RSAPrivateKey              @52
    _SSL_use_RSAPrivateKey_ASN1         @53
    _SSL_use_RSAPrivateKey_file         @54
    _SSL_use_certificate                @55
    _SSL_use_certificate_ASN1           @56
    _SSL_use_certificate_file           @57
    _d2i_SSL_SESSION                    @59
    _i2d_SSL_SESSION                    @60

