-----BEGIN CERTIFICATE-----
MIIBXDCCAQYCAQAwDQYJKoZIhvcNAQEEBQAwOTELMAkGA1UEBhMCQVUxFzAVBgNV
BAoTDkRvZGd5IEJyb3RoZXJzMREwDwYDVQQDEwhEb2RneSBDQTAeFw05NzAxMTMx
OTI0MTRaFw05NzAyMTIxOTI0MTRaMDkxCzAJBgNVBAYTAkFVMRcwFQYDVQQKEw5E
b2RneSBCcm90aGVyczERMA8GA1UEAxMIRG9kZ3kgQ0EwXDANBgkqhkiG9w0BAQEF
AANLADBIAkEAyN3476xcZaupyC9lkH9uA8eVLc6Mu7Fob0zdHYL+ZqB3YHB1xXg9
plzljDiINfLkq0zt0RWoIbfOt4yCcsXU/wIDAQABMA0GCSqGSIb3DQEBBAUAA0EA
sc/H0UxTOSWO92HjeBXwTGLByIVLPWXjjmusGsBGEWiD+CcuVswn94FsYJ/C0Gzs
5eaP3m1FyKw5fmzG7YqxRA==
-----END CERTIFICATE-----
