;
; Definition file for the DDL version of the CRYPT32 library from SSLeay
;

LIBRARY         CRYPT32

DESCRIPTION     'SSLeay CRYPT32 - eay@mincom.oz.au'

EXPORTS
    SSLeay                             @1
    SSLeay_version                     @2
    ASN1_BIT_STRING_asn1_meth          @600
    ASN1_HEADER_free                   @6
    ASN1_HEADER_new                    @7
    ASN1_IA5STRING_asn1_meth           @601
    ASN1_INTEGER_get                   @8
    ASN1_INTEGER_set                   @9
    ASN1_INTEGER_to_BN                 @10
    ASN1_OBJECT_free                   @11
    ASN1_OBJECT_new                    @12
    ASN1_PRINTABLE_type                @13
    ASN1_STRING_cmp                    @602
    ASN1_STRING_dup                    @603
    ASN1_STRING_free                   @604
    ASN1_STRING_new                    @605
    ASN1_STRING_print                  @606
    ASN1_STRING_set                    @607
    ASN1_STRING_type_new               @608
    ASN1_TYPE_free                     @15
    ASN1_TYPE_new                      @16
    ASN1_UTCTIME_check                 @17
    ASN1_UTCTIME_print                 @21
    ASN1_UTCTIME_set                   @22
    ASN1_check_infinite_end            @24
    ASN1_d2i_bio                       @25
    ASN1_d2i_fp                        @26
    ASN1_digest                        @638
    ASN1_dup                           @27
    ASN1_get_object                    @28
    ASN1_i2d_bio                       @29
    ASN1_i2d_fp                        @30
    ASN1_object_size                   @31
    ASN1_parse                         @32
    ASN1_put_object                    @33
    ASN1_sign                          @34
    ASN1_verify                        @35
    BF_cbc_encrypt                     @651
    BF_cfb64_encrypt                   @652
    BF_ecb_encrypt                     @653
    BF_encrypt                         @654
    BF_ofb64_encrypt                   @655
    BF_options                         @657
    BF_set_key                         @656
    BIO_ctrl                           @36
    BIO_ctrl_int                       @598
    BIO_debug_callback                 @37
    BIO_dump                           @640
    BIO_f_base64                       @615
    BIO_f_buffer                       @38
    BIO_f_cipher                       @616
    BIO_f_md                           @39
    BIO_fd_should_retry                @637
    BIO_find_type                      @650
    BIO_free                           @40
    BIO_free_all                       @649
    BIO_gets                           @41
    BIO_new                            @42
    BIO_pop                            @648
    BIO_printf                         @639
    BIO_push                           @647
    BIO_puts                           @43
    BIO_read                           @44
    BIO_s_fd                           @45
    BIO_s_file                         @46
    BIO_s_mem                          @47
    BIO_s_null                         @48
    BIO_s_socket                       @49
    BIO_set                            @588
    BIO_set_cipher                     @614
    BIO_sock_should_retry              @636
    BIO_write                          @50
    BN_CTX_free                        @51
    BN_CTX_new                         @52
    BN_MONT_CTX_free                   @53
    BN_MONT_CTX_new                    @54
    BN_MONT_CTX_set                    @55
    BN_add                             @56
    BN_add_word                        @57
    BN_bin2bn                          @58
    BN_bn2ascii                        @59
    BN_bn2bin                          @60
    BN_clear                           @61
    BN_clear_bit                       @62
    BN_clear_free                      @63
    BN_cmp                             @64
    BN_copy                            @65
    BN_div                             @66
    BN_dup                             @67
    BN_free                            @68
    BN_from_montgomery                 @69
    BN_gcd                             @70
    BN_generate_prime                  @71
    BN_is_bit_set                      @72
    BN_is_prime                        @73
    BN_lshift                          @74
    BN_lshift1                         @75
    BN_mask_bits                       @76
    BN_mod                             @77
    BN_mod_exp                         @78
    BN_mod_exp_mont                    @79
    BN_mod_exp_recp                    @80
    BN_mod_exp_simple                  @81
    BN_mod_inverse                     @82
    BN_mod_mul                         @83
    BN_mod_mul_montgomery              @84
    BN_mod_mul_reciprocal              @85
    BN_mod_word                        @86
    BN_mul                             @87
    BN_new                             @88
    BN_num_bits                        @89
    BN_num_bits_word                   @90
    BN_options                         @91
    BN_print                           @92
    BN_print_fp                        @93
    BN_rand                            @94
    BN_reciprocal                      @95
    BN_rshift                          @96
    BN_rshift1                         @97
    BN_set_bit                         @98
    BN_set_word                        @99
    BN_sqr                             @100
    BN_sub                             @101
    BN_to_ASN1_INTEGER                 @102
    BN_ucmp                            @103
    BN_value_one                       @104
    BUF_MEM_free                       @105
    BUF_MEM_grow                       @106
    BUF_MEM_new                        @107
    CERTIFICATE_CTX_free               @108
    CERTIFICATE_CTX_new                @109
    CONF_free                          @110
    CONF_get_number                    @111
    CONF_get_section                   @112
    CONF_get_string                    @113
    CONF_load                          @114
    CRYPTO_add_lock                    @582
    CRYPTO_dbg_free                    @645
    CRYPTO_dbg_malloc                  @643
    CRYPTO_dbg_realloc                 @644
    CRYPTO_dbg_remalloc                @596
    CRYPTO_free                        @591
    CRYPTO_get_add_lock_callback       @580
    CRYPTO_get_id_callback             @115
    CRYPTO_get_lock_name               @583
    CRYPTO_get_locking_callback        @116
    CRYPTO_get_mem_functions           @642
    CRYPTO_lock                        @117
    CRYPTO_malloc                      @589
    CRYPTO_mem_ctrl                    @592
    CRYPTO_mem_leaks                   @593
    CRYPTO_mem_leaks_cb                @646
    CRYPTO_mem_leaks_fp                @594
    CRYPTO_realloc                     @590
    CRYPTO_remalloc                    @595
    CRYPTO_set_add_lock_callback       @581
    CRYPTO_set_id_callback             @118
    CRYPTO_set_locking_callback        @119
    CRYPTO_set_mem_functions           @641
    CRYPTO_thread_id                   @120
    DH_check                           @121
    DH_compute_key                     @122
    DH_free                            @123
    DH_generate_key                    @124
    DH_generate_parameters             @125
    DH_new                             @126
    DH_size                            @127
    DHparams_print                     @128
    DHparams_print_fp                  @129
    DSA_free                           @130
    DSA_new                            @131
    DSA_print                          @132
    DSA_sign                           @133
    DSA_size                           @134
    DSA_verify                         @135
    ERR_clear_error                    @136
    ERR_error_string                   @137
    ERR_free_strings                   @138
    ERR_func_error_string              @139
    ERR_get_err_state_table            @140
    ERR_get_error                      @141
    ERR_get_error_line                 @142
    ERR_get_state                      @143
    ERR_get_string_table               @144
    ERR_lib_error_string               @145
    ERR_load_ASN1_strings              @146
    ERR_load_BN_strings                @147
    ERR_load_BUF_strings               @148
    ERR_load_CONF_strings              @149
    ERR_load_DH_strings                @150
    ERR_load_DSA_strings               @151
    ERR_load_ERR_strings               @152
    ERR_load_EVP_strings               @153
    ERR_load_OBJ_strings               @155
    ERR_load_PEM_strings               @156
    ERR_load_RSA_strings               @157
    ERR_load_X509_strings              @158
    ERR_load_crypto_strings            @159
    ERR_load_strings                   @160
    ERR_peek_error                     @161
    ERR_peek_error_line                @162
    ERR_print_errors                   @163
    ERR_print_errors_fp                @164
    ERR_put_error                      @165
    ERR_reason_error_string            @166
    ERR_remove_state                   @167
    EVP_BytesToKey                     @168
    EVP_CipherFinal                    @169
    EVP_CipherInit                     @170
    EVP_CipherUpdate                   @171
    EVP_DecodeBlock                    @172
    EVP_DecodeFinal                    @173
    EVP_DecodeInit                     @174
    EVP_DecodeUpdate                   @175
    EVP_DecryptFinal                   @176
    EVP_DecryptInit                    @177
    EVP_DecryptUpdate                  @178
    EVP_DigestFinal                    @179
    EVP_DigestInit                     @180
    EVP_DigestUpdate                   @181
    EVP_EncodeBlock                    @182
    EVP_EncodeFinal                    @183
    EVP_EncodeInit                     @184
    EVP_EncodeUpdate                   @185
    EVP_EncryptFinal                   @186
    EVP_EncryptInit                    @187
    EVP_EncryptUpdate                  @188
    EVP_OpenFinal                      @189
    EVP_OpenInit                       @190
    EVP_PKEY_free                      @191
    EVP_PKEY_new                       @192
    EVP_PKEY_size                      @193
    EVP_SealFinal                      @194
    EVP_SealInit                       @195
    EVP_SignFinal                      @196
    EVP_VerifyFinal                    @197
    EVP_bf_cbc                         @658
    EVP_bf_cfb                         @659
    EVP_bf_ecb                         @660
    EVP_bf_ofb                         @661
    EVP_des_cbc                        @198
    EVP_des_cfb                        @199
    EVP_des_ecb                        @200
    EVP_des_ede                        @201
    EVP_des_ede3                       @202
    EVP_des_ede3_cbc                   @203
    EVP_des_ede3_cfb                   @204
    EVP_des_ede3_ofb                   @205
    EVP_des_ede_cbc                    @206
    EVP_des_ede_cfb                    @207
    EVP_des_ede_ofb                    @208
    EVP_des_ofb                        @209
    EVP_desx_cbc                       @586
    EVP_dss                            @584
    EVP_dss1                           @585
    EVP_get_MDbyname                   @210
    EVP_get_cipherbyname               @211
    EVP_get_pw_prompt                  @212
    EVP_idea_cbc                       @213
    EVP_idea_cfb                       @214
    EVP_idea_ecb                       @215
    EVP_idea_ofb                       @216
    EVP_md2                            @217
    EVP_md5                            @218
    EVP_rc2_cbc                        @219
    EVP_rc2_cfb                        @220
    EVP_rc2_ecb                        @221
    EVP_rc2_ofb                        @222
    EVP_rc4                            @223
    EVP_read_pw_string                 @224
    EVP_set_pw_prompt                  @225
    EVP_sha                            @226
    EVP_sha1                           @227
    MD2                                @228
    MD2_Final                          @229
    MD2_Init                           @230
    MD2_Update                         @231
    MD2_options                        @232
    MD5                                @233
    MD5_Final                          @234
    MD5_Init                           @235
    MD5_Update                         @236
    NETSCAPE_SPKAC_free                @237
    NETSCAPE_SPKAC_new                 @238
    NETSCAPE_SPKI_free                 @239
    NETSCAPE_SPKI_new                  @240
    OBJ_bsearch                        @241
    OBJ_cmp                            @242
    OBJ_dup                            @243
    OBJ_ln2nid                         @244
    OBJ_nid2ln                         @245
    OBJ_nid2obj                        @246
    OBJ_nid2sn                         @247
    OBJ_obj2nid                        @248
    OBJ_sn2nid                         @249
    OBJ_txt2nid                        @250
    PEM_ASN1_read                      @251
    PEM_ASN1_read_bio                  @252
    PEM_ASN1_write                     @253
    PEM_ASN1_write_bio                 @254
    PEM_SealFinal                      @255
    PEM_SealInit                       @256
    PEM_SealUpdate                     @257
    PEM_SignFinal                      @258
    PEM_X509_INFO_read                 @259
    PEM_X509_INFO_read_bio             @260
    PEM_do_header                      @261
    PEM_get_EVP_CIPHER_INFO            @262
    PEM_read                           @263
    PEM_read_bio                       @264
    PEM_write                          @265
    PEM_write_bio                      @266
    PKCS7_DIGEST_free                  @267
    PKCS7_DIGEST_new                   @268
    PKCS7_ENCRYPT_free                 @269
    PKCS7_ENCRYPT_new                  @270
    PKCS7_ENC_CONTENT_free             @271
    PKCS7_ENC_CONTENT_new              @272
    PKCS7_ENVELOPE_free                @273
    PKCS7_ENVELOPE_new                 @274
    PKCS7_ISSUER_AND_SERIAL_free       @275
    PKCS7_ISSUER_AND_SERIAL_new        @276
    PKCS7_RECIP_INFO_free              @277
    PKCS7_RECIP_INFO_new               @278
    PKCS7_SIGNED_free                  @279
    PKCS7_SIGNED_new                   @280
    PKCS7_SIGNER_INFO_free             @281
    PKCS7_SIGNER_INFO_new              @282
    PKCS7_SIGN_ENVELOPE_free           @283
    PKCS7_SIGN_ENVELOPE_new            @284
    PKCS7_free                         @285
    PKCS7_new                          @286
    RAND_bytes                         @287
    RAND_cleanup                       @288
    RAND_file_name                     @289
    RAND_load_file                     @290
    RAND_screen                        @613
    RAND_seed                          @291
    RAND_write_file                    @292
    RC2_cbc_encrypt                    @293
    RC2_cfb64_encrypt                  @294
    RC2_ecb_encrypt                    @295
    RC2_encrypt                        @296
    RC2_ofb64_encrypt                  @297
    RC2_set_key                        @298
    RC4                                @299
    RC4_options                        @300
    RC4_set_key                        @301
    RSAPrivateKey_asn1_meth            @609
    RSA_free                           @302
    RSA_generate_key                   @303
    RSA_mod_exp                        @304
    RSA_new                            @305
    RSA_print                          @306
    RSA_print_fp                       @307
    RSA_private_decrypt                @308
    RSA_private_encrypt                @309
    RSA_public_decrypt                 @310
    RSA_public_encrypt                 @311
    RSA_sign                           @312
    RSA_size                           @313
    RSA_verify                         @314
    SHA                                @315
    SHA1                               @316
    SHA1_Final                         @317
    SHA1_Init                          @318
    SHA1_Update                        @319
    SHA_Final                          @320
    SHA_Init                           @321
    SHA_Update                         @322
    TXT_DB_create_index                @323
    TXT_DB_free                        @324
    TXT_DB_get_by_index                @325
    TXT_DB_insert                      @326
    TXT_DB_read                        @327
    TXT_DB_write                       @328
    X509_ALGOR_free                    @329
    X509_ALGOR_new                     @330
    X509_ATTRIBUTE_free                @331
    X509_ATTRIBUTE_new                 @332
    X509_CINF_free                     @333
    X509_CINF_new                      @334
    X509_CRL_INFO_free                 @335
    X509_CRL_INFO_new                  @336
    X509_CRL_free                      @337
    X509_CRL_new                       @338
    X509_EXTENSION_free                @577
    X509_EXTENSION_new                 @576
    X509_INFO_free                     @339
    X509_INFO_new                      @340
    X509_NAME_ENTRY_free               @341
    X509_NAME_ENTRY_new                @342
    X509_NAME_free                     @343
    X509_NAME_new                      @344
    X509_NAME_oneline                  @345
    X509_NAME_print                    @346
    X509_PKEY_free                     @347
    X509_PKEY_new                      @348
    X509_PUBKEY_free                   @349
    X509_PUBKEY_new                    @350
    X509_REQ_INFO_free                 @351
    X509_REQ_INFO_new                  @352
    X509_REQ_free                      @354
    X509_REQ_get_pubkey                @622
    X509_REQ_new                       @355
    X509_REQ_print                     @356
    X509_REQ_print_fp                  @357
    X509_REQ_set_pubkey                @621
    X509_REQ_set_subject_name          @618
    X509_REQ_set_version               @619
    X509_REQ_to_X509                   @358
    X509_REVOKED_free                  @359
    X509_REVOKED_new                   @360
    X509_SIG_free                      @361
    X509_SIG_new                       @362
    X509_VAL_free                      @363
    X509_VAL_new                       @364
    X509_add_cert                      @365
    X509_add_cert_dir                  @366
    X509_add_cert_file                 @367
    X509_asn1_meth                     @610
    X509_cert_verify                   @368
    X509_cert_verify_error_string      @369
    X509_check_private_key             @599
    X509_cmp_current_time              @370
    X509_free                          @372
    X509_get_cert                      @373
    X509_get_default_cert_area         @374
    X509_get_default_cert_dir          @375
    X509_get_default_cert_dir_env      @376
    X509_get_default_cert_file         @377
    X509_get_default_cert_file_env     @378
    X509_get_default_private_dir       @379
    X509_get_issuer_name               @630
    X509_get_pubkey                    @632
    X509_get_subject_name              @628
    X509_gmtime_adj                    @382
    X509_issuer_and_serial_cmp         @383
    X509_issuer_and_serial_hash        @384
    X509_issuer_name_cmp               @385
    X509_issuer_name_hash              @386
    X509_load_verify_locations         @387
    X509_name_cmp                      @388
    X509_name_hash                     @389
    X509_new                           @390
    X509_print                         @391
    X509_print_fp                      @392
    X509_set_default_verify_paths      @393
    X509_set_issuer_name               @629
    X509_set_notAfter                  @634
    X509_set_notBefore                 @635
    X509_set_pubkey                    @631
    X509_set_serialNumber              @623
    X509_set_subject_name              @627
    X509_set_version                   @625
    X509_subject_name_cmp              @394
    X509_subject_name_hash             @395
    X509_to_X509_REQ                   @396
    a2i_ASN1_INTEGER                   @397
    a2i_ASN1_STRING                    @611
    asn1_Finish                        @398
    asn1_GetSequence                   @399
    bn_div64                           @400
    bn_expand2                         @401
    bn_mul_add_word                    @402
    bn_mul_word                        @403
    bn_qadd                            @597
    bn_qsub                            @404
    bn_sqr_words                       @405
    crypt                              @406
    d2i_ASN1_BIT_STRING                @407
    d2i_ASN1_BOOLEAN                   @574
    d2i_ASN1_HEADER                    @408
    d2i_ASN1_IA5STRING                 @409
    d2i_ASN1_INTEGER                   @410
    d2i_ASN1_OBJECT                    @411
    d2i_ASN1_OCTET_STRING              @412
    d2i_ASN1_PRINTABLE                 @413
    d2i_ASN1_PRINTABLESTRING           @414
    d2i_ASN1_SET                       @415
    d2i_ASN1_T61STRING                 @416
    d2i_ASN1_TYPE                      @417
    d2i_ASN1_UTCTIME                   @418
    d2i_ASN1_bytes                     @419
    d2i_DHparams                       @420
    d2i_DSAPrivateKey                  @421
    d2i_DSAPublicKey                   @422
    d2i_NETSCAPE_SPKAC                 @423
    d2i_NETSCAPE_SPKI                  @424
    d2i_Netscape_RSA                   @425
    d2i_PKCS7                          @426
    d2i_PKCS7_DIGEST                   @427
    d2i_PKCS7_ENCRYPT                  @428
    d2i_PKCS7_ENC_CONTENT              @429
    d2i_PKCS7_ENVELOPE                 @430
    d2i_PKCS7_ISSUER_AND_SERIAL        @431
    d2i_PKCS7_RECIP_INFO               @432
    d2i_PKCS7_SIGNED                   @433
    d2i_PKCS7_SIGNER_INFO              @434
    d2i_PKCS7_SIGN_ENVELOPE            @435
    d2i_PrivateKey                     @436
    d2i_PublicKey                      @437
    d2i_RSAPrivateKey                  @438
    d2i_RSAPublicKey                   @439
    d2i_X509                           @440
    d2i_X509_ALGOR                     @441
    d2i_X509_ATTRIBUTE                 @442
    d2i_X509_CINF                      @443
    d2i_X509_CRL                       @444
    d2i_X509_CRL_INFO                  @445
    d2i_X509_EXTENSION                 @579
    d2i_X509_NAME                      @446
    d2i_X509_NAME_ENTRY                @447
    d2i_X509_PKEY                      @448
    d2i_X509_PUBKEY                    @449
    d2i_X509_REQ                       @450
    d2i_X509_REQ_INFO                  @451
    d2i_X509_REVOKED                   @452
    d2i_X509_SIG                       @453
    d2i_X509_VAL                       @454
    d2i_asn1_print_type                @455
    des_3cbc_encrypt                   @456
    des_cbc_cksum                      @457
    des_cbc_encrypt                    @458
    des_cblock_print_file              @459
    des_cfb64_encrypt                  @460
    des_cfb_encrypt                    @461
    des_decrypt3                       @633
    des_ecb3_encrypt                   @462
    des_ecb_encrypt                    @463
    des_ede3_cbc_encrypt               @464
    des_ede3_cfb64_encrypt             @465
    des_ede3_ofb64_encrypt             @466
    des_enc_read                       @467
    des_enc_write                      @468
    des_encrypt                        @469
    des_encrypt2                       @470
    des_encrypt3                       @617
    des_fcrypt                         @612
    des_is_weak_key                    @471
    des_key_sched                      @472
    des_ncbc_encrypt                   @473
    des_ofb64_encrypt                  @474
    des_ofb_encrypt                    @475
    des_options                        @476
    des_pcbc_encrypt                   @477
    des_quad_cksum                     @478
    des_random_key                     @479
    des_random_seed                    @480
    des_read_2passwords                @481
    des_read_password                  @482
    des_read_pw_string                 @483
    des_set_key                        @484
    des_set_odd_parity                 @485
    des_string_to_2keys                @486
    des_string_to_key                  @487
    des_xcbc_encrypt                   @587
    i2a_ASN1_INTEGER                   @488
    i2a_ASN1_OBJECT                    @489
    i2d_ASN1_BIT_STRING                @490
    i2d_ASN1_BOOLEAN                   @575
    i2d_ASN1_HEADER                    @491
    i2d_ASN1_IA5STRING                 @492
    i2d_ASN1_INTEGER                   @493
    i2d_ASN1_OBJECT                    @494
    i2d_ASN1_OCTET_STRING              @495
    i2d_ASN1_PRINTABLE                 @496
    i2d_ASN1_SET                       @497
    i2d_ASN1_TYPE                      @498
    i2d_ASN1_UTCTIME                   @499
    i2d_ASN1_bytes                     @500
    i2d_DHparams                       @501
    i2d_NETSCAPE_SPKAC                 @502
    i2d_NETSCAPE_SPKI                  @503
    i2d_Netscape_RSA                   @504
    i2d_PKCS7                          @505
    i2d_PKCS7_DIGEST                   @506
    i2d_PKCS7_ENCRYPT                  @507
    i2d_PKCS7_ENC_CONTENT              @508
    i2d_PKCS7_ENVELOPE                 @509
    i2d_PKCS7_ISSUER_AND_SERIAL        @510
    i2d_PKCS7_RECIP_INFO               @511
    i2d_PKCS7_SIGNED                   @512
    i2d_PKCS7_SIGNER_INFO              @513
    i2d_PKCS7_SIGN_ENVELOPE            @514
    i2d_PrivateKey                     @515
    i2d_PublicKey                      @516
    i2d_RSAPrivateKey                  @517
    i2d_RSAPublicKey                   @518
    i2d_X509                           @519
    i2d_X509_ALGOR                     @520
    i2d_X509_ATTRIBUTE                 @521
    i2d_X509_CINF                      @522
    i2d_X509_CRL                       @523
    i2d_X509_CRL_INFO                  @524
    i2d_X509_EXTENSION                 @578
    i2d_X509_NAME                      @525
    i2d_X509_NAME_ENTRY                @526
    i2d_X509_PKEY                      @527
    i2d_X509_PUBKEY                    @528
    i2d_X509_REQ                       @529
    i2d_X509_REQ_INFO                  @530
    i2d_X509_REVOKED                   @531
    i2d_X509_SIG                       @532
    i2d_X509_VAL                       @533
    idea_cbc_encrypt                   @534
    idea_cfb64_encrypt                 @535
    idea_ecb_encrypt                   @536
    idea_encrypt                       @537
    idea_ofb64_encrypt                 @538
    idea_options                       @539
    idea_set_decrypt_key               @540
    idea_set_encrypt_key               @541
    lh_delete                          @542
    lh_doall                           @543
    lh_doall_arg                       @544
    lh_free                            @545
    lh_insert                          @546
    lh_new                             @547
    lh_node_stats                      @548
    lh_node_stats_bio                  @549
    lh_node_usage_stats                @550
    lh_node_usage_stats_bio            @551
    lh_retrieve                        @552
    lh_stats                           @553
    lh_stats_bio                       @554
    lh_strhash                         @555
    sk_delete                          @559
    sk_delete_ptr                      @560
    sk_find                            @561
    sk_free                            @562
    sk_insert                          @563
    sk_new                             @564
    sk_pop                             @565
    sk_pop_free                        @566
    sk_push                            @567
    sk_shift                           @568
    sk_unshift                         @569

